==> rtaspi-0.1.29.tar.gz: rtaspi-0.1.29/src/rtaspi/device_managers/remote_desktop/rdp.py <==
"""RDP device implementation using FreeRDP."""

import logging
from typing import Optional, Dict, Any, Tuple
import subprocess
import threading
import time
import os
import tempfile

from .base import RemoteDesktopDevice
from ...core.logging import get_logger

logger = get_logger(__name__)

class RDPDevice(RemoteDesktopDevice):
    """RDP device implementation using FreeRDP as backend."""

    def __init__(self, device_id: str, config: Dict[str, Any]):
        """Initialize RDP device.
        
        Args:
            device_id: Unique device identifier
            config: Device configuration dictionary
        """
        super().__init__(device_id, config)
        
        # RDP specific configuration
        self.domain = config.get('domain', '')
        self.width = config.get('width', 1920)
        self.height = config.get('height', 1080)
        self.refresh_rate = config.get('refresh_rate', 30)
        
        # FreeRDP process management
        self._process: Optional[subprocess.Popen] = None
        self._capture_thread: Optional[threading.Thread] = None
        self._stop_capture = threading.Event()
        
        # Frame capture
        self._current_frame: Optional[bytes] = None
        self._frame_lock = threading.Lock()
        self._frame_ready = threading.Event()
        
        # Temporary file for sharing frames
        self._shared_memory_file = tempfile.NamedTemporaryFile(prefix='rtaspi_rdp_', delete=False)
        self._shared_memory_path = self._shared_memory_file.name

    def connect(self) -> bool:
        """Establish connection to RDP server using FreeRDP.
        
        Returns:
            bool: True if connection successful, False otherwise
        """
        try:
            # Build FreeRDP command
            cmd = [
                'xfreerdp',
                f'/v:{self.host}',
                f'/u:{self.username}',
                f'/p:{self.password}',
                f'/w:{self.width}',
                f'/h:{self.height}',
                '/cert-ignore',  # TODO: Proper certificate handling
                '/clipboard',
                f'/drive:temp,{os.path.dirname(self._shared_memory_path)}',
                '/jpeg',  # Enable JPEG compression
                '/jpeg-quality:80',
                '/gfx:RFX',  # Use RemoteFX codec
                '/gfx-progressive',  # Enable progressive codec
                '/smart-sizing',  # Enable smart sizing
                '/dynamic-resolution'  # Enable dynamic resolution
            ]

            if self.domain:
                cmd.append(f'/d:{self.domain}')

            # Start FreeRDP process
            self._process = subprocess.Popen(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE
            )

            # Start frame capture thread
            self._stop_capture.clear()
            self._capture_thread = threading.Thread(target=self._capture_frames)
            self._capture_thread.daemon = True
            self._capture_thread.start()

            # Wait for initial connection
            time.sleep(2)  # Give FreeRDP time to connect

            if self._process.poll() is not None:
                # Process terminated
                stderr = self._process.stderr.read().decode()
                logger.error(f"FreeRDP failed to start: {stderr}")
                return False

            self._connected = True
            logger.info(f"Connected to RDP server {self.host}")
            return True

        except Exception as e:
            logger.error(f"Failed to connect to RDP server: {e}")
            self.disconnect()
            return False

    def disconnect(self) -> None:
        """Disconnect from RDP server."""
        try:
            # Stop frame capture
            if self._capture_thread is not None:
                self._stop_capture.set()
                self._capture_thread.join(timeout=5)
                self._capture_thread = None

            # Terminate FreeRDP process
            if self._process is not None:
                self._process.terminate()
                try:
                    self._process.wait(timeout=5)
                except subprocess.TimeoutExpired:
                    self._process.kill()
                self._process = None

            # Clean up shared memory file
            try:
                os.unlink(self._shared_memory_path)
            except OSError:
                pass

            self._connected = False
            logger.info(f"Disconnected from RDP server {self.host}")

        except Exception as e:
            logger.error(f"Error during RDP disconnect: {e}")

    def _capture_frames(self) -> None:
        """Capture frames from FreeRDP session."""
        while not self._stop_capture.is_set():
            try:
                # TODO: Implement actual frame capture from FreeRDP
                # This is a placeholder - actual implementation will need to:
                # 1. Capture screen content from FreeRDP window
                # 2. Convert to appropriate format
                # 3. Store in self._current_frame
                # 4. Set frame_ready event
                time.sleep(1.0 / self.refresh_rate)
                
            except Exception as e:
                logger.error(f"Error capturing RDP frame: {e}")
                time.sleep(1)  # Avoid tight loop on error

    def get_frame(self) -> Optional[bytes]:
        """Get current frame from RDP session.
        
        Returns:
            Optional[bytes]: Frame data if available, None otherwise
        """
        with self._frame_lock:
            return self._current_frame

    def send_mouse_event(self, x: int, y: int, button: int = 0, pressed: bool = True) -> None:
        """Send mouse event to RDP session.
        
        Args:
            x: Mouse X coordinate
            y: Mouse Y coordinate
            button: Mouse button (0=left, 1=middle, 2=right)
            pressed: True if button pressed, False if released
        """
        if not self.is_connected:
            return

        try:
            # TODO: Implement mouse event sending through FreeRDP
            # This will require interfacing with FreeRDP's input system
            pass
        except Exception as e:
            logger.error(f"Failed to send mouse event: {e}")

    def send_key_event(self, key_code: int, pressed: bool = True) -> None:
        """Send keyboard event to RDP session.
        
        Args:
            key_code: Key code
            pressed: True if key pressed, False if released
        """
        if not self.is_connected:
            return

        try:
            # TODO: Implement keyboard event sending through FreeRDP
            # This will require interfacing with FreeRDP's input system
            pass
        except Exception as e:
            logger.error(f"Failed to send key event: {e}")

    def get_resolution(self) -> Tuple[int, int]:
        """Get current screen resolution.
        
        Returns:
            Tuple[int, int]: Width and height of RDP session
        """
        return self.width, self.height

    def set_resolution(self, width: int, height: int) -> bool:
        """Set screen resolution.
        
        Args:
            width: Desired screen width
            height: Desired screen height
            
        Returns:
            bool: True if resolution was set successfully
        """
        if not self.is_connected:
            return False

        try:
            # TODO: Implement dynamic resolution change through FreeRDP
            self.width = width
            self.height = height
            return True
        except Exception as e:
            logger.error(f"Failed to set resolution: {e}")
            return False

    def get_refresh_rate(self) -> int:
        """Get current refresh rate in Hz.
        
        Returns:
            int: Current refresh rate
        """
        return self.refresh_rate

    def set_refresh_rate(self, rate: int) -> bool:
        """Set refresh rate.
        
        Args:
            rate: Desired refresh rate in Hz
            
        Returns:
            bool: True if refresh rate was set successfully
        """
        if rate <= 0:
            return False
            
        try:
            self.refresh_rate = rate
            return True
        except Exception as e:
            logger.error(f"Failed to set refresh rate: {e}")
            return False

==> rtaspi-0.1.37.tar.gz: rtaspi-0.1.37/src/rtaspi/device_managers/remote_desktop/rdp.py <==
"""RDP device implementation using FreeRDP."""

import logging
from typing import Optional, Dict, Any, Tuple
import subprocess
import threading
import time
import os
import tempfile

from .base import RemoteDesktopDevice
from .capture import WindowCapture

logger = logging.getLogger(__name__)


class RDPDevice(RemoteDesktopDevice):
    """RDP device implementation using FreeRDP as backend."""

    def __init__(self, device_id: str, config: Dict[str, Any]):
        """Initialize RDP device.

        Args:
            device_id: Unique device identifier
            config: Device configuration dictionary
        """
        super().__init__(device_id, config)

        # RDP specific configuration
        self.domain = config.get("domain", "")
        self.width = config.get("width", 1920)
        self.height = config.get("height", 1080)
        self.refresh_rate = config.get("refresh_rate", 30)

        # FreeRDP process management
        self._process: Optional[subprocess.Popen] = None
        self._capture_thread: Optional[threading.Thread] = None
        self._stop_capture = threading.Event()

        # Frame capture
        self._current_frame: Optional[bytes] = None
        self._frame_lock = threading.Lock()
        self._frame_ready = threading.Event()
        self._window_capture = WindowCapture()

    def connect(self) -> bool:
        """Establish connection to RDP server using FreeRDP.

        Returns:
            bool: True if connection successful, False otherwise
        """
        try:
            # Build FreeRDP command
            cmd = [
                "xfreerdp",
                f"/v:{self.host}",
                f"/u:{self.username}",
                f"/p:{self.password}",
                f"/w:{self.width}",
                f"/h:{self.height}",
                "/cert-ignore",  # TODO: Proper certificate handling
                "/clipboard",
                "/jpeg",  # Enable JPEG compression
                "/jpeg-quality:80",
                "/gfx:RFX",  # Use RemoteFX codec
                "/gfx-progressive",  # Enable progressive codec
                "/smart-sizing",  # Enable smart sizing
                "/dynamic-resolution",  # Enable dynamic resolution
            ]

            if self.domain:
                cmd.append(f"/d:{self.domain}")

            # Start FreeRDP process
            self._process = subprocess.Popen(
                cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
            )

            # Start frame capture thread
            self._stop_capture.clear()
            self._capture_thread = threading.Thread(target=self._capture_frames)
            self._capture_thread.daemon = True
            self._capture_thread.start()

            # Wait for initial connection
            time.sleep(2)  # Give FreeRDP time to connect

            if self._process.poll() is not None:
                # Process terminated
                stderr = self._process.stderr.read().decode()
                logger.error(f"FreeRDP failed to start: {stderr}")
                return False

            self._connected = True
            logger.info(f"Connected to RDP server {self.host}")
            return True

        except Exception as e:
            logger.error(f"Failed to connect to RDP server: {e}")
            self.disconnect()
            return False

    def disconnect(self) -> None:
        """Disconnect from RDP server."""
        try:
            # Stop frame capture
            if self._capture_thread is not None:
                self._stop_capture.set()
                self._capture_thread.join(timeout=5)
                self._capture_thread = None

            # Terminate FreeRDP process
            if self._process is not None:
                self._process.terminate()
                try:
                    self._process.wait(timeout=5)
                except subprocess.TimeoutExpired:
                    self._process.kill()
                self._process = None

            # Clean up window capture
            if self._window_capture:
                self._window_capture.cleanup()

            self._connected = False
            logger.info(f"Disconnected from RDP server {self.host}")

        except Exception as e:
            logger.error(f"Error during RDP disconnect: {e}")

    def _capture_frames(self) -> None:
        """Capture frames from FreeRDP session."""
        # Initialize window capture
        if not self._window_capture.initialize():
            logger.error("Failed to initialize window capture")
            return

        # Wait for FreeRDP window to appear
        retries = 10
        while retries > 0 and not self._stop_capture.is_set():
            if self._window_capture.find_window_by_name("FreeRDP"):
                break
            retries -= 1
            time.sleep(0.5)

        if retries == 0:
            logger.error("Failed to find FreeRDP window")
            return

        # Capture frames
        while not self._stop_capture.is_set():
            try:
                frame = self._window_capture.capture_window()
                if frame:
                    with self._frame_lock:
                        self._current_frame = frame
                    self._frame_ready.set()
                time.sleep(1.0 / self.refresh_rate)

            except Exception as e:
                logger.error(f"Error capturing RDP frame: {e}")
                time.sleep(1)  # Avoid tight loop on error

    def get_frame(self) -> Optional[bytes]:
        """Get current frame from RDP session.

        Returns:
            Optional[bytes]: Frame data if available, None otherwise
        """
        with self._frame_lock:
            return self._current_frame

    def send_mouse_event(
        self, x: int, y: int, button: int = 0, pressed: bool = True
    ) -> None:
        """Send mouse event to RDP session.

        Args:
            x: Mouse X coordinate
            y: Mouse Y coordinate
            button: Mouse button (0=left, 1=middle, 2=right)
            pressed: True if button pressed, False if released
        """
        if not self.is_connected:
            return

        try:
            # TODO: Implement mouse event sending through FreeRDP
            # This will require interfacing with FreeRDP's input system
            pass
        except Exception as e:
            logger.error(f"Failed to send mouse event: {e}")

    def send_key_event(self, key_code: int, pressed: bool = True) -> None:
        """Send keyboard event to RDP session.

        Args:
            key_code: Key code
            pressed: True if key pressed, False if released
        """
        if not self.is_connected:
            return

        try:
            # TODO: Implement keyboard event sending through FreeRDP
            # This will require interfacing with FreeRDP's input system
            pass
        except Exception as e:
            logger.error(f"Failed to send key event: {e}")

    def get_resolution(self) -> Tuple[int, int]:
        """Get current screen resolution.

        Returns:
            Tuple[int, int]: Width and height of RDP session
        """
        return self.width, self.height

    def set_resolution(self, width: int, height: int) -> bool:
        """Set screen resolution.

        Args:
            width: Desired screen width
            height: Desired screen height

        Returns:
            bool: True if resolution was set successfully
        """
        if not self.is_connected:
            return False

        try:
            # TODO: Implement dynamic resolution change through FreeRDP
            self.width = width
            self.height = height
            return True
        except Exception as e:
            logger.error(f"Failed to set resolution: {e}")
            return False

    def get_refresh_rate(self) -> int:
        """Get current refresh rate in Hz.

        Returns:
            int: Current refresh rate
        """
        return self.refresh_rate

    def set_refresh_rate(self, rate: int) -> bool:
        """Set refresh rate.

        Args:
            rate: Desired refresh rate in Hz

        Returns:
            bool: True if refresh rate was set successfully
        """
        if rate <= 0:
            return False

        try:
            self.refresh_rate = rate
            return True
        except Exception as e:
            logger.error(f"Failed to set refresh rate: {e}")
            return False
